@@ -1,11 +1,14 @@
 /* uname.c: A replacement for the uname function (sort of)
 
-This software is Copyright 1996 by Craig Metz, All Rights Reserved. The
-Inner Net Copyright Notice and License Agreement applies to these
-portions of the software.
+%%% copyright-cmetz-96
+This software is Copyright 1996-1997 by Craig Metz, All Rights Reserved.
+The Inner Net License Version 2 applies to this software.
+You should have received a copy of the license with this software. If
+you didn't get a copy, you may request one from <license@inner.net>.
 
         History:
 
+	Modified by cmetz for OPIE 2.3. Ifdef around gethostname().
 	Created by cmetz for OPIE 2.2.
 */
 #include "opie_cfg.h"
@@ -16,6 +19,7 @@ portions of the software.
 
 int uname FUNCTION(struct utsname *buf)
 {
+#if HAVE_GETHOSTNAME
 	char hostname[MAXHOSTNAMELEN], *c;
 
 	memset(buf, 0, sizeof(buf));
@@ -31,4 +35,8 @@ int uname FUNCTION(struct utsname *buf)
 
 	strncpy(buf->nodename, hostname, sizeof(buf->nodename) - 1);
 	return 0;
+#else /* HAVE_GETHOSTNAME */
+	strncpy(buf->nodename, "unknown", sizeof(buf->nodename) - 1);
+	return 0;
+#endif /* HAVE_GETHOSTNAME */
 }
