@@ -1,9 +1,12 @@
 /* opie.h: Data structures and values for the OPIE authentication
 	system that a program might need.
 
-Portions of this software are Copyright 1996 by Craig Metz, All Rights
-Reserved. The Inner Net Copyright Notice and License Agreement applies to
-these portions of the software.
+%%% portions-copyright-cmetz-96
+Portions of this software are Copyright 1996-1997 by Craig Metz, All Rights
+Reserved. The Inner Net License Version 2 applies to these portions of
+the software.
+You should have received a copy of the license with this software. If
+you didn't get a copy, you may request one from <license@inner.net>.
 
 Portions of this software are Copyright 1995 by Randall Atkinson and Dan
 McDonald, All Rights Reserved. All Rights under this copyright are assigned
@@ -12,6 +15,12 @@ License Agreement applies to this software.
 
 	History:
 
+	Modified by cmetz for OPIE 2.31. Removed active attack protection.
+	Modified by cmetz for OPIE 2.3. Renamed PTR to VOIDPTR. Added
+		re-init key and extension file fields to struct opie. Added
+		opie_ prefix on struct opie members. Added opie_flags field
+		and definitions. Added more prototypes. Changed opiehash()
+		prototype.
 	Modified by cmetz for OPIE 2.22. Define __P correctly if this file
 		is included in a third-party program.
 	Modified by cmetz for OPIE 2.2. Re-did prototypes. Added FUNCTION
@@ -24,14 +33,14 @@ License Agreement applies to this software.
 		(skey.h).
 */
 #ifndef _OPIE_H
-#define _OPIE_H
+#define _OPIE_H 1
 
 #if _OPIE
 
 #if HAVE_VOIDPTR
-#define PTR void *
+#define VOIDPTR void *
 #else /* HAVE_VOIDPTR */
-#define PTR char *
+#define VOIDPTR char *
 #endif /* HAVE_VOIDPTR */
 
 #if HAVE_VOIDRET
@@ -70,6 +79,10 @@ struct utsname {
 	};
 #endif /* HAVE_SYS_UTSNAME_H */
 
+#ifndef _SC_OPEN_MAX
+#define _SC_OPEN_MAX 1
+#endif /* _SC_OPEN_MAX */
+
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 1024
 #endif /* MAXHOSTNAMELEN */
@@ -77,8 +90,10 @@ struct utsname {
 #else /* _OPIE */
 #ifdef __STDC__
 #define VOIDRET void
+#define VOIDPTR void *
 #else /* __STDC__ */
 #define VOIDRET
+#define VOIDPTR char *
 #endif /* __STDC__ */
 #endif /* _OPIE */
 
@@ -94,30 +109,28 @@ struct utsname {
 #endif /* __ARGS */
 #endif /* __P */
 
-/* Server library's internal state block.
-   Code outside libopie.a should treat this as an opaque data block and
-   should not ever manipulate this structure in any way. Its format is
-   subject to change in future versions of OPIE.
-*/
 struct opie {
-  void *keyfile;
-  char buf[256];
-  char *logname;
-  int n;
-  char *seed;
-  char *val;
-  long recstart;	/* needed so reread of buffer is efficient */
+  int opie_flags;
+  char opie_buf[256];
+  char *opie_principal;
+  int opie_n;
+  char *opie_seed;
+  char *opie_val;
+  long opie_recstart;
 };
 
+#define __OPIE_FLAGS_RW 1
+#define __OPIE_FLAGS_READ 2
+
 /* Minimum length of a secret password */
-#ifndef OPIE_PASS_MIN
-#define OPIE_PASS_MIN 10
-#endif	/* OPIE_PASS_MIN */
+#ifndef OPIE_SECRET_MIN
+#define OPIE_SECRET_MIN 10
+#endif	/* OPIE_SECRET_MIN */
 
 /* Maximum length of a secret password */
-#ifndef OPIE_PASS_MAX
-#define OPIE_PASS_MAX 127
-#endif	/* OPIE_PASS_MAX */
+#ifndef OPIE_SECRET_MAX
+#define OPIE_SECRET_MAX 127
+#endif	/* OPIE_SECRET_MAX */
 
 /* Minimum length of a seed */
 #ifndef OPIE_SEED_MIN
@@ -134,9 +147,9 @@ struct opie {
 #define OPIE_CHALLENGE_MAX (7+1+4+1+OPIE_SEED_MAX)
 #endif	/* OPIE_CHALLENGE_MAX */
 
-/* Maximum length of a response (six words up to four chars each w/spaces) */
+/* Maximum length of a response that we allow */
 #ifndef OPIE_RESPONSE_MAX
-#define OPIE_RESPONSE_MAX (6*4+5)
+#define OPIE_RESPONSE_MAX (9+1+19+1+9+OPIE_SEED_MAX+1+19+1+19+1+19)
 #endif	/* OPIE_RESPONSE_MAX */
 
 /* Maximum length of a principal (read: user name) */
@@ -168,17 +181,18 @@ int  opieaccessfile __P((char *));
 int  rdnets __P((long));
 int  isaddr __P((register char *));
 int  opiealways __P((char *));
-int  opieatob8 __P((char *,char *));
+char *opieatob8 __P((char *,char *));
 VOIDRET  opiebackspace __P((char *));
-int  opiebtoa8 __P((char *,char *));
-char * opiebtoe __P((char *,char *));
+char *opiebtoa8 __P((char *,char *));
+char *opiebtoe __P((char *,char *));
+char *opiebtoh __P((char *,char *));
 int  opieetob __P((char *,char *));
 int  opiechallenge __P((struct opie *,char *,char *));
 int  opiegenerator __P((char *,char *,char *));
 int  opiegetsequence __P((struct opie *));
-VOIDRET  opiehash __P((char *,unsigned));
+VOIDRET  opiehash __P((VOIDPTR, unsigned));
 int  opiehtoi __P((register char));
-int  opiekeycrunch __P((unsigned,char *,char *,char *));
+int  opiekeycrunch __P((int, char *, char *, char *));
 int  opielock __P((char *));
 int  opielookup __P((struct opie *,char *));
 VOIDRET  opiemd4init __P((struct opiemdx_ctx *));
@@ -192,6 +206,17 @@ VOIDRET  opierandomchallenge __P((char *));
 char * opieskipspace __P((register char *));
 VOIDRET  opiestripcrlf __P((char *));
 int  opieverify __P((struct opie *,char *));
-int opiepasswd __P((struct opie *, char *, int, char *, char *));
+int opiepasswd __P((struct opie *, int, char *, int, char *, char *));
+char *opiereadpass __P((char *, int, int));
+int opielogin __P((char *line, char *name, char *host));
 
+#if _OPIE
+struct utmp;
+int __opiegetutmpentry __P((char *, struct utmp *));
+#ifdef EOF
+FILE *__opieopen __P((char *, int, int));
+#endif /* EOF */
+int __opiereadrec __P((struct opie *));
+int __opiewriterec __P((struct opie *));
+#endif /* _OPIE */
 #endif /* _OPIE_H */
